//----------------------------------------------------------------------------
//  Header: CHistoCollection.h, v 1.0                         2008/11/19    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <cstdlib>

#include "TObject.h"
#include "TFile.h"
#include "TList.h"
#include "TH1.h"
#include "TH2.h"

#include "CHistoCollection.h"

using namespace std;

CHistoCollection::CHistoCollection(TFile *_fp)
{
   fout              = _fp;
   fHistoList        = new TList();
   // create lists
   // 1 - pack results
   // 2 - general histograms
   // 3 - pickup related
   // 4 - far trigger related
   // 5 - near trigger related
   for(int n=0; n<5; ++n) {
    fHistoListArrResults.push_back(new TList());
   }
   // fHistoList->SetOwner(true);  // necessary to delete object in the list in destructor -- error in ROOT
   CreateClientHistograms();
}

CHistoCollection::~CHistoCollection()
{
   cout << "calling CHistoCollection::~CHistoCollection() " << flush << endl;

   cout << "Info from <CHistoCollection::~CHistoCollection()>: Deleting list " << flush << endl;
   // delete List
   if(fHistoList) {
      delete fHistoList;
      fHistoList = NULL;
   }
   for(unsigned int n=0; n<fHistoListArrResults.size(); ++n) {
     if(fHistoListArrResults[n]) {
        delete fHistoListArrResults[n];
        fHistoListArrResults[n] = NULL;
     }
   }
   fHistoListArrResults.clear();
}

void CHistoCollection::SaveListToFile()
{
   if(!fout) {
      cerr << "Error in <DHist::SaveListToFile>: File does not exist" << flush << endl;
      exit(-1);
   }
   // GO TO Proper file
   fout->cd();
   
   // make subfolder -- packs, fits and events distributions from gas
   // analysis library
   //
   fout->mkdir("SingleDistributions")->cd();
   TObjLink *lnk = fHistoList->FirstLink();
   while(lnk) {
      TObject *tmp = lnk->GetObject();
      tmp->Write();
      lnk = lnk->Next();
   }
   fout->cd();
   // make subfolder for analysis results
   std::string folderName[] = {"PackResults", 
                               "GeneralDistribution", 
                               "PickupDistributions", 
                               "FarDistributions", 
                               "NearDistributions"};
   for(unsigned int n=0; n<fHistoListArrResults.size(); ++n) {
     if(n<(sizeof(folderName)/sizeof(string))) {
       fout->mkdir(folderName[n].c_str())->cd();
     } else {
       fout->mkdir("AverageDistributions");
     }
     lnk = fHistoListArrResults[n]->FirstLink();
     while(lnk) {
       TObject *tmp = lnk->GetObject();
       tmp->Write();
       lnk = lnk->Next();
     }
     fout->cd();
   }
}


void CHistoCollection::AddObjectToBeSaved(TObject *_save)
{
   if(fHistoList && _save)
      fHistoList->Add(_save);
   else
      cerr << "Error in <CHistoCollection::AddObjectToBeSaved>: Attempting to add NULL pointer to list" << flush << endl;
}

void CHistoCollection::AddObjectToListOfResults(TObject *_save, const unsigned int _position)
{
   if(_position<fHistoListArrResults.size()) {
     if(fHistoListArrResults[_position] && _save)
       fHistoListArrResults[_position]->Add(_save);
     else
       cerr << "Error in <CHistoCollection::AddObjectToListOfResults>: Attempting to add NULL pointer to list" << flush << endl;
   }
}

void  CHistoCollection::CreateClientHistograms()
{
   double HVFieldMin = -12000.0;
   double HVFieldMax = -8000.0;
   // pickup part
   hAmplitudeVsPeakPos = new TH2D("hAmplitudeVsPeakPos", "amplitude vs peak pos.", 512, 0.0, 512.0, 1024, 0.0, 1024.0);
   AddObjectToListOfResults(hAmplitudeVsPeakPos, 2);

   hWidthVsPeakPos = new TH2D("hWidthVsPeakPos","width vs peak pos.", 512, 0.0, 512.0, 100, 0.0, 200.0);
   AddObjectToListOfResults(hWidthVsPeakPos, 2);

   hOffsetVsPeakPos = new TH2D("hOffsetVsPeakPos","offset vs peak pos.", 512, 0.0, 512.0, 100, 0.0, 200.0);
   AddObjectToListOfResults(hOffsetVsPeakPos,2 );
   
   hChi2VsPeakPos = new TH2D("hChi2VsPeakPos","chi^{2} vs peak pos.", 512, 0.0, 512.0, 250, 0.0, 250.0);
   AddObjectToListOfResults(hChi2VsPeakPos, 2);
   
   hAreaVsPeakPos = new TH2D("hAreaVsPeakPos","Area vs peak pos.", 512, 0.0, 512.0, 1500, 0.0, 15000.0);
   AddObjectToListOfResults(hAreaVsPeakPos, 2);

   hAmplitudeVsAreaPickup = new TH2D("hAmplitudeVsAreaPickup","Amplitude vs area from Pickup.", 1500, 0.0, 15000.0, 1024, 0.0, 1024.0);
   AddObjectToListOfResults(hAmplitudeVsAreaPickup, 2);
   
   hPOverTemp1VsPickupPeakPosition = new TH2D("hPOverTemp1VsPickupPeakPosition", "P/T1 vs pickup position", 512, 0.0, 512.0, 1000, 2.0, 4.0);
   AddObjectToListOfResults(hPOverTemp1VsPickupPeakPosition, 2);

   hPOverTemp2VsPickupPeakPosition = new TH2D("hPOverTemp2VsPickupPeakPosition", "P/T2 vs pickup position", 512, 0.0, 512.0, 1000, 2.0, 4.0);
   AddObjectToListOfResults(hPOverTemp2VsPickupPeakPosition, 2);



   // trigger near
   hNearAmplitudeVsPickupPeakPosition = new TH2F("hNearAmplitudeVsPickupPeakPosition","Amplitude Near vs peak position Pickup", 1024, 0.0, 1024,  512, 0.0, 512.0);
   AddObjectToListOfResults(hNearAmplitudeVsPickupPeakPosition, 4);
  
   hNearTotalAmplitudeVsPickupPeakPosition = new TH2F("hNearTotalAmplitudeVsPickupPeakPosition","(Amplitude+Offset) Near vs peak position Pickup", 1024, 0.0, 1024,  512, 0.0, 512.0);
   AddObjectToListOfResults(hNearTotalAmplitudeVsPickupPeakPosition, 4);

   hNearAreaVsPickupPeakPosition  = new TH2F("hNearAreaVsPickupPeakPosition","Area Near vs peak position Pickup",  512, 0.0, 512.0, 1500, 0.0, 15000.0);
   AddObjectToListOfResults(hNearAreaVsPickupPeakPosition, 4);
   
   hNearAreaVsPickupArea = new TH2F("hNearAreaVsPickupArea","Area Near vs area Pickup", 1500, 0.0, 15000.0,  1500, 0.0, 15000.0);
   AddObjectToListOfResults(hNearAreaVsPickupArea, 4);

   hNearPositionVsPickupPeakPosition = new TH2F("hNearPositionVsPickupPeakPosition","Near peak position vs Pickup peak position",  512, 0.0, 512.0,  512, 0.0, 512.0);
   AddObjectToListOfResults(hNearPositionVsPickupPeakPosition, 4);

   hNearWidthVsPickupPeakPosition = new TH2F("hNearWidthVsPickupPeakPosition","Near peak width vs Pickup peak position", 512, 0.0, 512.0, 100, 0.0, 100.0);
   AddObjectToListOfResults(hNearWidthVsPickupPeakPosition, 4);

   hNearAmplitudeVsNearArea = new TH2F("hNearAmplitudeVsNearArea","Near amplitude vs Near area", 1024, 0.0, 1024, 1500, 0.0, 15000.0);
   AddObjectToListOfResults(hNearAmplitudeVsNearArea, 4);


   
   // trigger far
   hFarAmplitudeVsPickupPeakPosition = new TH2F("hFarAmplitudeVsPickupPeakPosition","Amplitude Far vs Pickup peak position", 1024, 0.0, 1024,  512, 0.0, 512.0);
   AddObjectToListOfResults(hFarAmplitudeVsPickupPeakPosition, 3);

   hFarTotalAmplitudeVsPickupPeakPosition = new TH2F("hFarTotalAmplitudeVsPickupPeakPosition","(Amplitude+Offset) Far vs Pickup peak position", 1024, 0.0, 1024,  512, 0.0, 512.0);
   AddObjectToListOfResults(hFarTotalAmplitudeVsPickupPeakPosition, 3);

   hFarAreaVsPickupPeakPosition = new TH2F("hFarAreaVsPickupPeakPosition","Area Far vs peak position Pickup", 512, 0.0, 512.0, 1500, 0.0, 15000.0);
   AddObjectToListOfResults(hFarAreaVsPickupPeakPosition, 3);
   
   hFarAreaVsPickupArea = new TH2F("hFarAreaVsPickupArea","Area Far vs area Pickup", 1500, 0.0, 15000.0,  1500, 0.0, 15000.0);
   AddObjectToListOfResults(hFarAreaVsPickupArea, 3);
  
   hFarPositionVsPickupPeakPosition = new TH2F("hFarPositionVsPickupPeakPosition","Far peak position vs Pickup peak position",  512, 0.0, 512.0,  512, 0.0, 512.0);
   AddObjectToListOfResults(hFarPositionVsPickupPeakPosition, 3);
 
   hFarWidthVsPickupPeakPosition = new TH2F("hFarWidthVsPickupPeakPosition","Far peak width vs Pickup peak position", 512, 0.0, 512.0, 100, 0.0, 100.0);
   AddObjectToListOfResults(hFarWidthVsPickupPeakPosition, 3);
   
   hFarAmplitudeVsFarArea = new TH2F("hFarAmplitudeVsFarArea","Far amplitude vs Far area", 1500, 0.0, 15000.0, 1024, 0.0, 1024);
   AddObjectToListOfResults(hFarAmplitudeVsFarArea, 3);



   // trigger correlation
   hNearOffsetVsFarOffset = new TH2F("hNearOffsetVsFarOffset","Near peak offset vs Far peak offest", 200, 0.0, 200.0, 200, 0.0, 200.0);
   AddObjectToListOfResults(hNearOffsetVsFarOffset, 1);


   
   // general
   hPOverTemp1VsEField = new TH2D("hPOverTemp1VsEField", "P/T1 vs EField", 1000, HVFieldMin, HVFieldMax, 500, 2.0, 4.0);
   AddObjectToListOfResults(hPOverTemp1VsEField, 1);

   hPOverTemp2VsEField = new TH2D("hPOverTemp2VsEField", "P/T2 vs EField", 1000, HVFieldMin, HVFieldMax, 500, 2.0, 4.0);
   AddObjectToListOfResults(hPOverTemp2VsEField, 1);

   hTemp1VsEField = new TH2D("hTemp1VsEField", "T1 vs EField", 1000, HVFieldMin, HVFieldMax, 100, 250.0, 350.0);
   AddObjectToListOfResults(hTemp1VsEField, 1);

   hTemp2VsEField = new TH2D("hTemp2VsEField", "T2 vs EField", 1000, HVFieldMin, HVFieldMax, 100, 250.0, 350.0);
   AddObjectToListOfResults(hTemp2VsEField, 1);

   hTemp1VsTemp2 = new TH2F("hTemp1VsTemp2","T1 vs T2", 100, 250.0, 350.0, 100, 250.0, 350.0);
   AddObjectToListOfResults(hTemp1VsTemp2, 1);
      
   hErrPeakPosition = new TH1D("hErrPeakPosition","Fit error of Pcikup position", 800, -0.5, 5.5);
   AddObjectToListOfResults(hErrPeakPosition, 1);

   hTriggerElecVsTriggerSoft = new TH2F("hTriggerElecVsTriggerSoft","Trigger Bit from Electronic vs Trigger extacted from spectrum", 5, -0.5, 4.5, 5, -0.5, 4.5);
   AddObjectToListOfResults(hTriggerElecVsTriggerSoft, 1);

   hNumberOfEventsPerPack = new TH1F("hNumberOfEventsPerPack","hNumberOfEventsPerPack", 1500, 0.0, 3000.0);
   AddObjectToListOfResults(hNumberOfEventsPerPack, 1);

   hDriftVelocityResolution = new TH1D("hDriftVelocityResolution","# of counts vs drift velocity", 10000, -0.5, 10.5);
   AddObjectToListOfResults(hDriftVelocityResolution, 1);
   
   hNearGainResolution = new TH1D("hNearGainResolution","# of counts vs near gain", 30000, 0.5, 9000.5);
   AddObjectToListOfResults(hNearGainResolution, 1);
   
   hFarGainResolution = new TH1D("hFarGainResolution","# of counts vs far gain", 30000, 0.5, 9000.5);
   AddObjectToListOfResults(hFarGainResolution, 1);
}


void CHistoCollection::CreateTimeDependendHistograms(Int_t _NumberOfPacks)
{
   // pack results
   hPickupNearAArea = new TH1D("hPickupNearAArea","Mean Pickup Area from Near Events vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hPickupNearAArea, 0);
   
   hPickupFarAArea  = new TH1D("hPickupFarAArea","Mean Pickup Area from Far Events vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hPickupFarAArea, 0);
   
   hNearGainArea = new TH1D("hNearGainArea","Gain from Near vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hNearGainArea, 0);
   
   hFarGainArea  = new TH1D("hFarGainArea","Gain from Far vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hFarGainArea, 0);
   
   hNearGainAreaCorrected = new TH1D("hNearGainAreaCorrected","Corrected gain from Near vs pack number",  _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hNearGainAreaCorrected, 0);
   
   hFarGainAreaCorrected  = new TH1D("hFarGainAreaCorrected","Corrected gain from Far vs pack number",  _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hFarGainAreaCorrected, 0);
   
   hDriftVelocity = new TH1D("hDriftVelocity","Drift velocity vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hDriftVelocity, 0);

   hDriftVelocityCorrected = new TH1D("hDriftVelocityCorrected","Corrected drift velocity vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hDriftVelocityCorrected, 0);
   
   hFieldCageHV = new TH1D("hFieldCageHV","Drift voltage vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hFieldCageHV, 0);

   hPickupHV = new TH1D("hPickupHV","Pickup anode voltage vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hPickupHV, 0);
   
   hTempSensor1 = new TH1D("hTempSensor1","Temperature from sensor 1 vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hTempSensor1, 0);

   hTempSensor2 = new TH1D("hTempSensor2","Temperature from sensor 2 vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hTempSensor2, 0);
   
   hPressure = new TH1D("hPressure","Atmospheric pressure vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hPressure, 0);
   
   hFractionCO2 = new TH1D("hFractionCO2","CO2 concentraction vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hFractionCO2, 0);

   hFractionN2 = new TH1D("hFractionN2","N2 concentraction vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hFractionN2, 0);
   
   hCorrectionFactor_DriftVelocity = new TH1D("hCorrectionFactor_DriftVelocity","Correction factor for drift velocity vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hCorrectionFactor_DriftVelocity, 0);
      
   hCorrectionFactor_P_TU = new TH1D("hCorrectionFactor_P_TU","Correction factor #frac{P}{T*U} vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hCorrectionFactor_P_TU, 0);

   hCorrectionFactor_P_T = new TH1D("hCorrectionFactor_P_T","Correction factor #frac{P}{T} vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hCorrectionFactor_P_T, 0);

   hPackAquizitionTime = new  TH1D("hPackAquizitionTime","Pack aquizition time vs pack number", _NumberOfPacks, 0.5, double(_NumberOfPacks)+0.5);
   AddObjectToListOfResults(hPackAquizitionTime, 0);
}
