//----------------------------------------------------------------------------
//  Header: CHistoCollection.h, v 1.0                         2008/11/19    //
//                                                                          //
//  Goofie ALICE Program Library                                            //
//  Copyright (C) ALICE collaboration, 2008                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#ifndef CHISTOCOLLECTION
#define CHISTOCOLLECTION

#include <deque>

class TFile;
class TObject;
class Tlist;
class TH1D;
class TH2D;

class CHistoCollection
{
public:
   CHistoCollection(TFile *_fp);
   ~CHistoCollection();
   
   void     SaveListToFile();
   void     AddObjectToBeSaved(TObject *_save);
   void     AddObjectToListOfResults(TObject *_save, const unsigned int _position = 0);
   void     CreateClientHistograms();
   void     CreateTimeDependendHistograms(Int_t _NumberOfPacks);

public:
   // list of client histograms
   // pickup
   TH2D    *hAmplitudeVsPeakPos;
   TH2D    *hWidthVsPeakPos;
   TH2D    *hOffsetVsPeakPos;
   TH2D    *hChi2VsPeakPos;
   TH2D    *hAreaVsPeakPos;
   TH2D    *hAmplitudeVsAreaPickup;
   TH2D    *hPOverTemp1VsPickupPeakPosition;
   TH2D    *hPOverTemp2VsPickupPeakPosition;

   
   // trigger near
   TH2F    *hNearAmplitudeVsPickupPeakPosition;
   TH2F    *hNearTotalAmplitudeVsPickupPeakPosition;
   TH2F    *hNearAreaVsPickupPeakPosition;
   TH2F    *hNearAreaVsPickupArea;
   TH2F    *hNearPositionVsPickupPeakPosition;
   TH2F    *hNearWidthVsPickupPeakPosition;
   TH2F    *hNearAmplitudeVsNearArea;

   // trigger far
   TH2F    *hFarAmplitudeVsPickupPeakPosition;
   TH2F    *hFarTotalAmplitudeVsPickupPeakPosition;
   TH2F    *hFarAreaVsPickupPeakPosition;
   TH2F    *hFarAreaVsPickupArea;
   TH2F    *hFarPositionVsPickupPeakPosition;
   TH2F    *hFarWidthVsPickupPeakPosition;
   TH2F    *hFarAmplitudeVsFarArea;

   // trigger correlation
   TH2F    *hNearOffsetVsFarOffset;

   // general
   TH2D    *hPOverTemp1VsEField;
   TH2D    *hPOverTemp2VsEField;
   TH2D    *hTemp1VsEField;
   TH2D    *hTemp2VsEField;
   TH2F    *hTemp1VsTemp2;
   TH1D    *hErrPeakPosition;
   TH2F    *hTriggerElecVsTriggerSoft;
   TH1F    *hNumberOfEventsPerPack;
   TH1D    *hDriftVelocityResolution;
   TH1D    *hFarGainResolution;
   TH1D    *hNearGainResolution;

   
   // pack results
   TH1D    *hPickupNearAArea;
   TH1D    *hPickupFarAArea;
   TH1D    *hNearGainArea;
   TH1D    *hFarGainArea;
   TH1D    *hNearGainAreaCorrected;
   TH1D    *hFarGainAreaCorrected;
   TH1D    *hDriftVelocity;
   TH1D    *hDriftVelocityCorrected;
   TH1D    *hFieldCageHV;
   TH1D    *hPickupHV;
   TH1D    *hTempSensor1;
   TH1D    *hTempSensor2;
   TH1D    *hPressure;
   TH1D    *hFractionCO2;
   TH1D    *hFractionN2;
   TH1D    *hCorrectionFactor_DriftVelocity;
   TH1D    *hCorrectionFactor_P_TU;
   TH1D    *hCorrectionFactor_P_T;
   TH1D    *hPackAquizitionTime;

private:
   TFile                *fout;
   TList                *fHistoList;
   std::deque<TList *>   fHistoListArrResults;
};
#endif
