//----------------------------------------------------------------------------
//  Body: AliGoofieProcessROOTFiles.cxx, v 1.0                2009/10/20    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2009                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include <iostream>
#include <cstdlib>
#include <list>
#include <unistd.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <TString.h>
#include <TROOT.h>
#include <TFile.h>
#include <TTree.h>

#include "AliGoofiePack.h"
#include "AliGoofieProcessROOTFiles.h"

using namespace std;

list<TString> *AliGoofieProcessROOTFiles::TakeListOfFiles(TString  *fpathName)
{
   DIR            *dir          = NULL;
   struct  dirent *dir_cont     = NULL;
   struct  stat    st;
   list<TString>  *flistOfFiles = new list<TString>;

   TString fname     = "/";
   // check for '/' in the fpathName
   if(fname.CompareTo(fpathName->operator[](fpathName->Length()-1))!=0) {
      fpathName->Append("/");
   }
   dir = opendir(fpathName->Data());
   if(dir) {
      while((dir_cont=readdir(dir))!=NULL) {
         fname = *fpathName + dir_cont->d_name;
         lstat(fname.Data(), &st);
         if(S_ISREG(st.st_mode) || S_ISLNK(st.st_mode)) {
	    // check if the name contain .root extension
	    if(fname.Index(".root")>=0) {
	       flistOfFiles->push_back(fname);
	    }
         }
      }
      closedir(dir);
      //set list in increasing order
      flistOfFiles->sort();
   } else {
      cerr << "Error in <AliGoofieProcessROOTFiles::TakeListOfFiles>: list of file is missing " << flush << endl;
      return NULL;
   }
   return flistOfFiles;
}

unsigned long AliGoofieProcessROOTFiles::GetTotalNumberOfPackForAnalysis(list<TString> *_FileList,
									 AliGoofiePack *_fpack,
									 const int _PacksPerEvents)
{
  // save current directory
  TDirectory    *fSavedHisto              = gDirectory;
  unsigned long  numberOfEventsPerPack    = 0;
  unsigned long  numberOfPacks            = 0;

  //loop over input files
  for (list<TString>::iterator it = _FileList->begin(); it != _FileList->end(); ++it) {
    // get input file
    TString inputFile = *it;
    cout << "Input file = " << inputFile << endl;
    //Open File
    TFile *lfile = new TFile(inputFile.Data(), "READ");
    if(lfile->IsZombie()) {
      cerr << "Error in <AliGoofieProcessROOTFiles::GetTotalNumberOfPackForAnalysis>: Fill is not readable, skipping file" << flush << endl;
      if(lfile) {
	delete lfile;
	lfile = NULL;
      }
      continue;
    }
    TTree *ltree = (TTree*)lfile->Get("T");
    ltree->SetBranchAddress("Packs", &_fpack);
    ltree->SetBranchStatus("*", 1);
    // number of pack store in ROOT files
    numberOfPacks += ltree->GetEntries();

    // get more details
    if(_PacksPerEvents>0) {
      for (Int_t ev=0; ev<ltree->GetEntries(); ++ev) {
	// *************** event loop ***************
	// for every pack one need to clean memory first -- ROOT way of processing events
	_fpack->Clear();
	
	if(ltree->GetEvent(ev) < 0) {
	  cerr << "Error in <AliGoofieProcessROOTFiles::GetTotalNumberOfPackForAnalysis>:Wrong event number = " <<  ev << endl; 
	  continue;
	}
	// fill list
	numberOfEventsPerPack += _fpack->GetNumberOfEvents();
      }
    }
    // close and delete file
    if(lfile) {
      lfile->Close();
      delete lfile;
      lfile = NULL;
    }
  }
  // set main directory
  gDirectory = fSavedHisto;

  // return value
  if(_PacksPerEvents<=0) {
    if(numberOfPacks) {
      // if number of event pre pack not specified return number of packs in root files
      return numberOfPacks;
    } else {
      return 1;
    }
  } else {
    return (unsigned long)(1+((float)numberOfEventsPerPack/(float)_PacksPerEvents));
  }
}
