//----------------------------------------------------------------------------
//  Body: CErrorMonitor.cpp, v 1.0                            2009/07/30    //
//                                                                          //
//  Goofie TPC ALICE Program Library                                        //
//  Copyright (C) ALICE collaboration, 2009                                 //
//                                                                          //
//  Last modified in CVS                                                    //
//  Author: D. Antonczyk                                                    //
//----------------------------------------------------------------------------

#include "CErrorMonitor.h"


CErrorMonitor::CErrorMonitor() 
{
  fBusConnection  = NULL;
  fBusMessage     = NULL;
 
  fPathName       = "/dbus/alice/goofie/tpc";
  fInterfaceName  = "dbus.alice.goofie.tpc";
  fsPackNumber    = "0";
}


CErrorMonitor::~CErrorMonitor() 
{
  //
  dbus_error_free (&fBusError);

  // free dbus respources
  if(fBusMessage) {
    dbus_message_unref (fBusMessage);
  }

  // 
  if(fBusConnection)
  dbus_connection_unref(fBusConnection);
}


bool CErrorMonitor::Initialization()
{
  try {
    // initialize error object
    dbus_error_init(&fBusError);

    // get connection for user session
    fBusConnection = dbus_bus_get(DBUS_BUS_SESSION, &fBusError);
    if (!fBusConnection)  throw (int)-91001000;

    // check whether required name already reserved. 
    dbus_bool_t ret = dbus_bus_name_has_owner(fBusConnection, fInterfaceName, &fBusError);
    if ( dbus_error_is_set(&error) )  throw (int)-91001001;

    // if not request it
    if( ret == FALSE ) {
      int request_name_reply = dbus_bus_request_name( connection,name, DBUS_NAME_FLAG_DO_NOT_QUEUE, &error);
      
      if ( dbus_error_is_set(&error) )  throw (int)-91001002;

      if ( request_name_reply != DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER ) throw (int)-91001003;
    }
  } catch (int _thrFlag) {

    if(_thrFlag -91001000)
      cerr << "Error in <CErrorMonitor::Initialization()>::Failed to connect to the D-BUS daemon: %s", fBusError.message << flush << endl;
    
    if(_thrFlag -91001001)
      cerr << "Error in <CErrorMonitor::Initialization()>::D-BUS error: ", fBusError.message << flush << endl;
    
    if(_thrFlag -91001002)
      cerr << "Error in <CErrorMonitor::Initialization()>::Failed to reserved name: ", fBusError.message << flush << endl;
    
    // 
    dbus_error_free(&fBusError);
    return false;
  }
  return true;
}



void CErrorMonitor::SendRunningStateConfirmation() throw int
{
  if(fBusConnection) throw (int)-0x91000010;

  fBusMessage = dbus_message_new_signal (fPathName.c_str(),
					 fInterfaceName.c_str(), 
					 "RunningStateConfirmation");
  
  const char * sPackNumber = fsPackNumber.c_str();

  /* Append the argument to the message, must ends with DBUS_TYPE_INVALID */
  dbus_message_append_args(fBusMessage,
			   DBUS_TYPE_STRING, 
			   &sPackNumber,
			   DBUS_TYPE_INVALID);
  // Send the signal
  dbus_connection_send (fBusConnection, fBusMessage, NULL);
  // Decrements the reference count of a DBusMessage, 
  // freeing the message if the count reaches 0.
  dbus_message_unref (fBusMessage);
}
